<?php

/**
 * @file
 * The "Drupal Credentials" page layout.
 */

namespace Drupal\desk_net\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Component\Utility\Crypt;

class DrupalCredentialsController extends ControllerBase {

  protected $api_key;

  protected $api_secret;

  public function __construct() {
    $config = \Drupal::config('desk_net.settings');
    $this->api_key = $config->get('desk_net_api_key');
    $this->api_secret = $config->get('desk_net_api_secret');
  }

  /**
   * Generating custom form for "Drupal Credentials" page.
   */
  public function drupalCredentials() {
    $host = \Drupal::request()->getSchemeAndHttpHost();

    $html = '<h2>' . \Drupal::translation()->translate('Drupal Credentials') . '</h2>';
    $html .= '<p>';
    $html .= \Drupal::translation()->translate('Use these credentials in Desk-Net on the Advanced Settings tab of 
   the <a href="https://www.desk-net.com/objectsPage.htm" target="_blank">
   platform</a> you are connecting to this Drupal website.');
    $html .= '</p>';

    $html .= '<table class="form-table">
                <tbody>
                    <tr>
                        <th scope="row">' . \Drupal::translation()->translate('API URL') . '</th>
                        <td>' . $host . '/dr-json/v1</td>
                    </tr>
                    <tr>
                        <th scope="row">' . \Drupal::translation()->translate('API User') . '</th>
                        <td id="api_key">
                        ' . $this->api_key . '
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">' . \Drupal::translation()->translate('API Secret') . '</th>
                        <td id="api_secret">
                        ' . $this->api_secret . '
                        </td>
                    </tr>
                 </tbody>
             </table>';

    // Modal dialog.
    $html .= '<div id="consentModal" class="modal">
                <!-- Modal content -->
                <div class="modal-content">
                    <span class="close">&times;</span>
                    <p>' . \Drupal::translation()->translate('Are you sure to generate new credentials?') . '</p>
                    <div class="position-control-element"><a id="confirm" href="' . $host . '/admin/config/desk-net/generate-new-credentials" class="button form-submit
                    btn-primary-submit" value="Confirm">Confirm</a>
                    <input type="button" name="cancel" id="cancel"
                    class="button form-submit" value="Cancel"></div>
                </div>
              </div>';

    $html .= '<input type="button" id="generate-new-credentials-submit"
  value="Generate new credentials" class="button form-submit">';

    $this->showMessage();

    return [
      '#allowed_tags' => [
        'input',
        'table',
        'tbody',
        'tr',
        'td',
        'th',
        'div',
        'span',
        'p',
        'h2',
        'a',
      ],
      '#markup' => $html,
    ];
  }

  /**
   * Showing message after successfully generate new credentials.
   */
  private function showMessage() {
    // Getting GET parameters from url and check on generate new credentials.
    $request = \Drupal::request();
    $dn_credentials = $request->query->get('dn-credentials');

    if ($dn_credentials === 'generate') {
      \Drupal::messenger()
        ->addStatus(\Drupal::translation()->translate('New credentials successfully generated.'));
    }
  }

  /**
   * Perform generate Drupal token.
   *
   * @return string $status_list
   */
  public static function generateDrupalModuleToken(): string {
    $token = Crypt::randomBytesBase64(32);
    // Hash the token using SHA-256.
    $token_hash = hash('sha256', $token);

    self::addTokeToList($token_hash);

    return $token;
  }

  /**
   * Performing remove expired tokens.
   */
  public static function removeExpiredTokens() {
    $current_date_time = date('Y-m-d H:i:s');

    $query = \Drupal::database()
      ->delete('desk_net_tokens')
      ->condition('expired_date', $current_date_time, '<')
      ->execute();
  }

  /**
   * Performing add a new token.
   */
  public static function addTokeToList($token) {
    $current_date_time = date('Y-m-d H:i:s');
    $expiration_date = date('Y-m-d H:i:s', strtotime($current_date_time . ' + 5 minutes'));
    $table_name = 'desk_net_tokens';

    $database = \Drupal::database();
    $query = $database->insert($table_name)
      ->fields([
        'token' => $token,
        'expired_date' => $expiration_date,
      ])
      ->execute();
  }

}