<?php

namespace Drupal\desk_net\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\EventDispatcher\Event;
use Drupal\node\NodeInterface;

use Drupal\desk_net\Collection\NoticesCollection;
use Drupal\desk_net\Controller\ModuleSettings;
use Drupal\desk_net\DeleteMethods;
use Drupal\desk_net\Controller\ActionController;
use Drupal\desk_net\Event\EntityUpdateEvent;

class DeskNetSubscriber implements EventSubscriberInterface {

  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  public static function getSubscribedEvents() {
    return [
      EntityUpdateEvent::EVENT_NAME => 'processApiRequest',
    ];
  }

  /**
   * Builds and executes API requests for the given entity and action.
   *
   * @param EntityUpdateEvent $event
   *   The entity for which the API request is being processed.
   */
  public function processApiRequest(EntityUpdateEvent $event) {
    $entity = $event->entity;
    $action = $event->action;
    // Getting Desk-Net revision data.
    $desk_net_revision = ModuleSettings::deskNetRevisionGet($entity);

    switch ($action) {
      case 'insert':
        if (ModuleSettings::checkUserCredentials() && empty($desk_net_revision)) {
          ActionController::createNodeDrupalToDN($entity);
        }
        else {
          \Drupal::messenger()->addError(NoticesCollection::getNotice(5));
        }

        break;
      case 'update':
        if (ModuleSettings::checkUserCredentials()) {
          if ($desk_net_revision !== FALSE && !empty($desk_net_revision['desk_net_story_id'])) {
            ActionController::updateNodeDrupalToDN($entity, $entity->bundle());
          }
          else {
            ActionController::createNodeDrupalToDN($entity);
          }
        }
        else {
          \Drupal::messenger()->addError(NoticesCollection::getNotice(5));
        }

        break;
      case 'delete':
        try {
          $new_entity = \Drupal::entityTypeManager()
            ->getStorage($entity->getEntityTypeId())
            ->load($entity->id());
          if (!$new_entity) {
            (new DeleteMethods())->deleteElement($entity);
          }
        }
        catch (\Drupal\Component\Plugin\Exception\PluginNotFoundException $e) {
          (new DeleteMethods())->deleteElement($entity);
        }
    }
  }

}