<?php

/**
 * @file
 * The "Slug syncing" form.
 */

namespace Drupal\desk_net\Form;

use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\desk_net\Controller\ModuleSettings;
use Drupal\desk_net\Collection\NoticesCollection;
use Drupal\desk_net\PageTemplate\PageTemplate;

class DeskNetSlugSyncing extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'desk_net_slug_syncing';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $link_slug_feature = Link::fromTextAndUrl(\Drupal::translation()
      ->translate('slug feature'), Url::fromUri("http://support.desk-net.com/hc/en-us/articles/115003545092", ['attributes' => ['target' => '_blank']]))
      ->toString();

    // Slug field Desk-Net.
    $desk_net_slug['slug']['id'] = 'slug_syncing';
    $desk_net_slug['slug']['name'] = \Drupal::translation()
      ->translate('Slug (Desk-Net)');

    // Set values.
    $desk_net_slug_value['desk_net_slug_permalink_val']['id'] = 'url_alias';
    $desk_net_slug_value['desk_net_slug_permalink_val']['name'] = \Drupal::translation()
      ->translate('URL Alias');
    $desk_net_slug_value['desk_net_slug_headline_val']['id'] = 'title';
    $desk_net_slug_value['desk_net_slug_headline_val']['name'] = \Drupal::translation()
      ->translate('Title');

    $html = '<h2>' . \Drupal::translation()
        ->translate('Slug Syncing') . '</h2>';
    $html .= '<p>';
    $html .= \Drupal::translation()
      ->translate('Desk-Net syncs the Title, SEO Title and the URL Alias with the 
      @link in Desk-Net. Please define what field should be primarily synced to 
      Desk-Net in case of conflicts.', ['@link' => $link_slug_feature]);
    $html .= '</p>';

    $form['html'] = [
      '#markup' => $html,
    ];

    $form['desk_net_slug_syncing'] = PageTemplate::desk_net_matching_page_template(
      'desk_net_slug_syncing', $desk_net_slug, $desk_net_slug_value, '', '', 'url_alias');

    $initial_slug_syncing = (bool) \Drupal::config('desk_net.settings')
      ->get('initial_slug_syncing') ?? FALSE;

    $form['initial_slug_syncing'] = [
      '#type' => 'checkbox',
      '#title' => \Drupal::translation()
        ->translate('Only allow initial slug syncing'),
      '#default_value' => $initial_slug_syncing,
      '#required' => FALSE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => \Drupal::translation()->translate('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $formValues = NULL;

    if (!empty($form_state->getValues())) {
      foreach ($form_state->getValues() as $key => $value) {
        if ($key != 'form_id' && $key != 'op' && $key != 'form_token' &&
          $key != 'form_build_id' && $key != 'submit'
        ) {
          // Save value.
          $formValues[$key] = $value;
        }
      }

      $initial_slug_syncing = $form_state->getValue('initial_slug_syncing');

      // Save the values to configuration.
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_slug_syncing', serialize($formValues))
        ->set('initial_slug_syncing', $initial_slug_syncing)
        ->save();

      \Drupal::messenger()->addStatus(NoticesCollection::getNotice(13));
    }
  }

}